package cache;

import java.applet.*;
import java.awt.*;
import eduni.simanim.*;
import eduni.simjava.*;
import eduni.simdiag.*;

class Cache extends Sim_entity {
  private Sim_port frames[];
  private Sim_port mem, cpu;
  private int assoc, hits, requests;

  public Cache(String name, int assoc, int num_frames, int x, int y) {
    super(name, "cache", x, y);
    cpu = new Sim_port("cpu", "port", Anim_port.TOP, 2); add_port(cpu);
    mem = new Sim_port("mem", "port", Anim_port.BOTTOM, 2); add_port(mem);
    frames = new Sim_port[num_frames];
    for(int i=0; i<frames.length; i++) {
      frames[i] = new Sim_port("frame_"+i, "port", Anim_port.RIGHT, 8*i);
      add_port(frames[i]);
    }
    this.frames = frames;
    this.assoc = assoc;
    hits = 0;
    requests = 0;
    add_param(new Anim_param("Hits", Anim_param.NAME_VALUE, "0", -103, 20));
    add_param(new Anim_param("Hit ratio", Anim_param.NAME_VALUE, "0", -103, 36));
  }

  public void body() {
    int address, free, base_frame, i;
    boolean found;
    Sim_event ev = new Sim_event();

    while(true) {
      sim_get_next(ev); // get mem request from Cpu
      sim_hold(0.01);
      address = ev.get_tag();
      requests++;

      // Ask frames for address
      base_frame = (address % (frames.length / assoc)) * assoc;
      found = false;
      free = -1;
      for(i=base_frame; i<(base_frame+assoc); i++) {
        sim_schedule(frames[i], 0.0, 1, new Integer(address));
        sim_trace(1, "S frame_"+i+" R");
      }
      for(i=0; i<assoc; i++) {
        sim_get_next(ev);
        if(ev.get_tag() == 1) found = true;        // record that we have found addr
        if(ev.get_tag() == 0) free = i+base_frame; // record suitable replacement
      }
      sim_hold(0.01);
      if(!found) {
        // Fetch from memory
        sim_schedule(mem, 0.0, address);
        sim_trace(1, "S mem "+address);
        sim_get_next(ev);
        sim_hold(0.01);
        // Update cache
        if(free == -1) // If there are no free frames then pick one at random
          free = ((int)Math.floor(base_frame + assoc*Math.random()));
        sim_schedule(frames[free], 0.0, 0, new Integer(address));
        sim_trace(1, "S frame_"+free+" W");   
      } else {
        hits++;
      }
      sim_trace(1, "P "+hits+" "+((float)hits/(float)requests));
      // send it to the Cpu
      sim_schedule(cpu, 0.0, address);
      sim_trace(1, "S cpu "+address);
    }
  }
}
