/**
 * A work farm example, with integrated timing diagram
 */

package workfarm;

import java.applet.*;
import java.awt.*;
import java.awt.event.*;
import eduni.simanim.*;
import eduni.simjava.*;
import eduni.simdiag.*;


// The main applet
public class Applet5 extends Anim_applet implements ItemListener {
  Panel inputs,outputs;
  Label msgLabel, sinkLabel, srcLabel, numWorkLabel, showMsgLabel;
  TextField msgField, sinkField, srcField;
  Choice numWorkChoice;
  Checkbox showMsgBox;
  static public boolean show_msg = true;

  // initialise the gui inputs
  public void anim_init() {
    int i;
    inputs = new Panel();
    outputs = new Panel();
    GridBagLayout gb = new GridBagLayout();
    GridBagConstraints c = new GridBagConstraints();
    outputs.setLayout(gb);
    inputs.setLayout(new GridLayout(0,2));

    msgLabel = new Label("Number of jobs:", Label.RIGHT);
    inputs.add(msgLabel);
    msgField = new TextField("5",3);
    inputs.add(msgField);
    
 
    numWorkLabel = new Label("Number of workers:", Label.RIGHT);
    inputs.add(numWorkLabel);
    numWorkChoice = new Choice();
    for(i=1; i<=5; i++) numWorkChoice.addItem(" "+i+" ");
    numWorkChoice.select(2);
    numWorkChoice.addItemListener(this);;
    inputs.add(numWorkChoice);

    showMsgLabel = new Label("Show messages:", Label.RIGHT);
    inputs.add(showMsgLabel);
    showMsgBox = new Checkbox("");
    showMsgBox.setState(true);
    inputs.add(showMsgBox);

    srcLabel = new Label("Farmer delay between jobs (mean):", Label.RIGHT);
    inputs.add(srcLabel);
    srcField = new TextField("0.4",5);
    inputs.add(srcField);

    sinkLabel = new Label("Worker processing delay (mean):", Label.RIGHT);
    inputs.add(sinkLabel);
    sinkField = new TextField("1.0",5);
    inputs.add(sinkField);

    /* Place inputs and timing diagram into gridbag */
    c.fill = GridBagConstraints.BOTH;
    c.gridwidth = GridBagConstraints.REMAINDER;
    c.weighty = 1.0;
    outputs.add(inputs); gb.setConstraints(inputs,c);

    // Add a timing diagram
    TimingWindow tw = new TimingWindow();
    trace_out.addTraceListener( tw.getDiag() );
    tw.start();

    // Add another timing diagram
    TimingWindow tw2 = new TimingWindow();
    trace_out.addTraceListener( tw2.getDiag() );
    // Add a trace saver
    tw2.getDiag().addTraceListener( new TraceSaver("tracefile") );
    tw2.start();

 
    /* add all to the top level gridbag layout */
    this.add("North", outputs); //layout.setConstraints(outputs, gbc); 
  }

  // Setup the animation
  public void anim_layout() {
    int num_msg, i, num_workers;
    double src_hold, sink_hold;

    System.out.println("Laying out simulation");
    num_msg = Integer.parseInt(msgField.getText());
    num_workers = numWorkChoice.getSelectedIndex()+1;
    src_hold = Double.valueOf(srcField.getText()).doubleValue();
    sink_hold = Double.valueOf(sinkField.getText()).doubleValue();
    show_msg = showMsgBox.getState();

    // Add entities
    Sim_system.add(new Farmer("Farmer", num_msg, src_hold, 20, 20));
    Sim_system.add(new Bucket("Bucket_1", 90+70*num_workers, 20, Anim_port.LEFT));
    Sim_system.add(new Bucket("Bucket_2", 20, 160, Anim_port.RIGHT));
    for(i=1; i<=num_workers; i++) {
      Sim_system.add(new Wrouter("Wrouter_"+i, 20+70*i, 20));
      Sim_system.add(new Worker("Worker_"+i, sink_hold, 20+70*i, 90));
      Sim_system.add(new Rrouter("Rrouter_"+i, 20+70*i, 160));
    }

    // Link entities
    Sim_system.link_ports("Farmer", "first", "Wrouter_1", "in");
    Sim_system.link_ports("Wrouter_"+num_workers, "next", "Bucket_1", "in");
    Sim_system.link_ports("Rrouter_1", "out", "Bucket_2", "in");
    for(i=1; i<=num_workers; i++) {
      if(i!=num_workers)
        Sim_system.link_ports("Wrouter_"+i, "next", "Wrouter_"+(i+1), "in");
      Sim_system.link_ports("Wrouter_"+i, "worker", "Worker_"+i, "in");
      Sim_system.link_ports("Worker_"+i, "out", "Rrouter_"+i, "in1");
      if(i!=1)
        Sim_system.link_ports("Rrouter_"+i, "out", "Rrouter_"+(i-1), "in2");
    }
  }

  // do a layout if they change the number of workers
  public void itemStateChanged(ItemEvent e) {
    Object source = e.getSource();
    if (source == numWorkChoice)
        anim_relayout();
  }
}
